`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:24:55 03/09/2011 
// Design Name: 
// Module Name:    adc_data_input 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

module adc_data_input(
    // Differential Serial Data Inputs
    input [15:0] G1AD_N,
    input [15:0] G1AD_P,
    input [15:0] G2AD_N,
    input [15:0] G2AD_P,
    input [15:0] G3AD_N,
    input [15:0] G3AD_P,
    input [15:0] G4AD_N,
    input [15:0] G4AD_P,
    input [15:0] G5AD_N,
    input [15:0] G5AD_P,
    input [15:0] G6AD_N,
    input [15:0] G6AD_P,
    // Differential Frame Clock Inputs
    input G1ADCLK0N,
    input G1ADCLK0P,
    input G1ADCLK1N,
    input G1ADCLK1P,
    input G2ADCLK0N,
    input G2ADCLK0P,
    input G2ADCLK1N,
    input G2ADCLK1P,
    input G3ADCLK0N,
    input G3ADCLK0P,
    input G3ADCLK1N,
    input G3ADCLK1P,
    input G4ADCLK0N,
    input G4ADCLK0P,
    input G4ADCLK1N,
    input G4ADCLK1P,
    input G5ADCLK0N,
    input G5ADCLK0P,
    input G5ADCLK1N,
    input G5ADCLK1P,
    input G6ADCLK0N,
    input G6ADCLK0P,
    input G6ADCLK1N,
    input G6ADCLK1P,
    // Differential Bit Clock Inputs
    input G1LCLK0N,
    input G1LCLK0P,
    input G1LCLK1N,
    input G1LCLK1P,
    input G2LCLK0N,
    input G2LCLK0P,
    input G2LCLK1N,
    input G2LCLK1P,
    input G3LCLK0N,
    input G3LCLK0P,
    input G3LCLK1N,
    input G3LCLK1P,
    input G4LCLK0N,
    input G4LCLK0P,
    input G4LCLK1N,
    input G4LCLK1P,
    input G5LCLK0N,
    input G5LCLK0P,
    input G5LCLK1N,
    input G5LCLK1P,
    input G6LCLK0N,
    input G6LCLK0P,
    input G6LCLK1N,
    input G6LCLK1P,
	 // Clocks and Resets
	 input RST,
	 input CLK20,

	 output G1FRM_CLK0,      // Frame Clock for G1 ADC0 
	 output G1FRM_CLK1,      // Frame Clock for G1 ADC1 
	 output G2FRM_CLK0,      // Frame Clock for G2 ADC0 
	 output G2FRM_CLK1,      // Frame Clock for G2 ADC1 
	 output G3FRM_CLK0,      // Frame Clock for G3 ADC0
	 output G3FRM_CLK1,      // Frame Clock for G3 ADC1 
	 output G4FRM_CLK0,      // Frame Clock for G4 ADC0 
	 output G4FRM_CLK1,      // Frame Clock for G4 ADC1 
	 output G5FRM_CLK0,      // Frame Clock for G5 ADC0
	 output G5FRM_CLK1,      // Frame Clock for G5 ADC1 
	 output G6FRM_CLK0,      // Frame Clock for G6 ADC0 
	 output G6FRM_CLK1,      // Frame Clock for G6 ADC1 
    // Deserialized outputs - 16 channels of 12 bits flattend into single vectors (channel 15 in high order bits, 0 in low)
	 output DSR_ALIGNED,
    output [191:0] G1DAQ16CH,
    output [191:0] G2DAQ16CH,
    output [191:0] G3DAQ16CH,
    output [191:0] G4DAQ16CH,
    output [191:0] G5DAQ16CH,
    output [191:0] G6DAQ16CH
    );

  assign LA_G5QODDCH0 = g5qodd[0];
  assign LA_G5QEVNCH0 = g5qevn[0];
  assign LA_G3QODDCH0 = g3qodd[0];
  assign LA_G3QEVNCH0 = g3qevn[0];
  assign LA_G5BIT_SLIP_ODD0 = g5bit_slip_odd0;
  assign LA_G3BIT_SLIP_ODD0 = g3bit_slip_odd0;
  assign LA_G5BIT_SLIP_EVN0 = g5bit_slip_evn0;
  assign LA_G3BIT_SLIP_EVN0 = g3bit_slip_evn0;

  wire [15:0] g1ad,g2ad,g3ad,g4ad,g5ad,g6ad;
  wire [15:0] g1ad_b,g2ad_b,g3ad_b,g4ad_b,g5ad_b,g6ad_b;
  wire [15:0] g1ad_d,g2ad_d,g3ad_d,g4ad_d,g5ad_d,g6ad_d;
  wire [15:0] g1ad_db,g2ad_db,g3ad_db,g4ad_db,g5ad_db,g6ad_db;
  
  wire g1adclk0,g1adclk1;
  wire g2adclk0,g2adclk1;
  wire g3adclk0,g3adclk1;
  wire g4adclk0,g4adclk1;
  wire g5adclk0,g5adclk1;
  wire g6adclk0,g6adclk1;
  wire g1lclk0,g1lclk1,g1bitclk0,g1bitclk1;
  wire g2lclk0,g2lclk1,g2bitclk0,g2bitclk1;
  wire g3lclk0,g3lclk1,g3bitclk0,g3bitclk1;
  wire g4lclk0,g4lclk1,g4bitclk0,g4bitclk1;
  wire g5lclk0,g5lclk1,g5bitclk0,g5bitclk1;
  wire g6lclk0,g6lclk1,g6bitclk0,g6bitclk1;
  
  
  reg g1bit_slip_odd0, g1bit_slip_evn0;
  reg g1bit_slip_odd1, g1bit_slip_evn1;
  reg g2bit_slip_odd0, g2bit_slip_evn0;
  reg g2bit_slip_odd1, g2bit_slip_evn1;
  reg g3bit_slip_odd0, g3bit_slip_evn0;
  reg g3bit_slip_odd1, g3bit_slip_evn1;
  reg g4bit_slip_odd0, g4bit_slip_evn0;
  reg g4bit_slip_odd1, g4bit_slip_evn1;
  reg g5bit_slip_odd0, g5bit_slip_evn0;
  reg g5bit_slip_odd1, g5bit_slip_evn1;
  reg g6bit_slip_odd0, g6bit_slip_evn0;
  reg g6bit_slip_odd1, g6bit_slip_evn1;
  
  reg g1rst0;
  reg g1rst1;
  reg g2rst0;
  reg g2rst1;
  reg g3rst0;
  reg g3rst1;
  reg g4rst0;
  reg g4rst1;
  reg g5rst0;
  reg g5rst1;
  reg g6rst0;
  reg g6rst1;
  
//  wire g1a_clk,g2a_clk,g1_frm_clk_src;
//  wire g3a_clk,g4a_clk,g3_frm_clk_src;
//  wire g5a_clk,g6a_clk,g5_frm_clk_src;
  
  wire rdy;
  
  wire [4:0] g1cntout,g2cntout,g3cntout,g4cntout,g5cntout,g6cntout;
  wire [11:0] g1dout [15:0];
  wire [11:0] g2dout [15:0];
  wire [11:0] g3dout [15:0];
  wire [11:0] g4dout [15:0];
  wire [11:0] g5dout [15:0];
  wire [11:0] g6dout [15:0];
  
  

  assign G1DAQ16CH = {g1dout[15],g1dout[14],g1dout[13],g1dout[12],
                      g1dout[11],g1dout[10],g1dout[9],g1dout[8],
							 g1dout[7],g1dout[6],g1dout[5],g1dout[4],
							 g1dout[3],g1dout[2],g1dout[1],g1dout[0]};
  assign G2DAQ16CH = {g2dout[15],g2dout[14],g2dout[13],g2dout[12],
                      g2dout[11],g2dout[10],g2dout[9],g2dout[8],
							 g2dout[7],g2dout[6],g2dout[5],g2dout[4],
							 g2dout[3],g2dout[2],g2dout[1],g2dout[0]};
  assign G3DAQ16CH = {g3dout[15],g3dout[14],g3dout[13],g3dout[12],
                      g3dout[11],g3dout[10],g3dout[9],g3dout[8],
							 g3dout[7],g3dout[6],g3dout[5],g3dout[4],
							 g3dout[3],g3dout[2],g3dout[1],g3dout[0]};
  assign G4DAQ16CH = {g4dout[15],g4dout[14],g4dout[13],g4dout[12],
                      g4dout[11],g4dout[10],g4dout[9],g4dout[8],
							 g4dout[7],g4dout[6],g4dout[5],g4dout[4],
							 g4dout[3],g4dout[2],g4dout[1],g4dout[0]};
  assign G5DAQ16CH = {g5dout[15],g5dout[14],g5dout[13],g5dout[12],
                      g5dout[11],g5dout[10],g5dout[9],g5dout[8],
							 g5dout[7],g5dout[6],g5dout[5],g5dout[4],
							 g5dout[3],g5dout[2],g5dout[1],g5dout[0]};
  assign G6DAQ16CH = {g6dout[15],g6dout[14],g6dout[13],g6dout[12],
                      g6dout[11],g6dout[10],g6dout[9],g6dout[8],
							 g6dout[7],g6dout[6],g6dout[5],g6dout[4],
							 g6dout[3],g6dout[2],g6dout[1],g6dout[0]};
							 
  wire [5:0] g1qodd [15:0];
  wire [5:0] g2qodd [15:0];
  wire [5:0] g3qodd [15:0];
  wire [5:0] g4qodd [15:0];
  wire [5:0] g5qodd [15:0];
  wire [5:0] g6qodd [15:0];
  
  wire [5:0] g1qevn [15:0];
  wire [5:0] g2qevn [15:0];
  wire [5:0] g3qevn [15:0];
  wire [5:0] g4qevn [15:0];
  wire [5:0] g5qevn [15:0];
  wire [5:0] g6qevn [15:0];
  
	wire bit_slip_odd;
	wire bit_slip_evn;
	wire wrst;
	wire winc;
	reg [2:0] wait_count;
	reg [3:0] evn_count;
	reg [3:0] odd_count;
  
  
   IBUFDS_DIFF_OUT #(.DIFF_TERM("TRUE"),.IBUF_LOW_PWR("TRUE"),.IOSTANDARD("DEFAULT"))
	IBUFDS_DIFF_OUT_G1AD[15:0] (.O(g1ad),.OB(g1ad_b),.I(G1AD_P),.IB(G1AD_N));
  
   IBUFDS_DIFF_OUT #(.DIFF_TERM("TRUE"),.IBUF_LOW_PWR("TRUE"),.IOSTANDARD("DEFAULT"))
	IBUFDS_DIFF_OUT_G2AD[15:0] (.O(g2ad),.OB(g2ad_b),.I(G2AD_P),.IB(G2AD_N));
  
   IBUFDS_DIFF_OUT #(.DIFF_TERM("TRUE"),.IBUF_LOW_PWR("TRUE"),.IOSTANDARD("DEFAULT"))
	IBUFDS_DIFF_OUT_G3AD[15:0] (.O(g3ad),.OB(g3ad_b),.I(G3AD_P),.IB(G3AD_N));
  
   IBUFDS_DIFF_OUT #(.DIFF_TERM("TRUE"),.IBUF_LOW_PWR("TRUE"),.IOSTANDARD("DEFAULT"))
	IBUFDS_DIFF_OUT_G4AD[15:0] (.O(g4ad),.OB(g4ad_b),.I(G4AD_P),.IB(G4AD_N));
  
   IBUFDS_DIFF_OUT #(.DIFF_TERM("TRUE"),.IBUF_LOW_PWR("TRUE"),.IOSTANDARD("DEFAULT"))
	IBUFDS_DIFF_OUT_G5AD[15:0] (.O(g5ad),.OB(g5ad_b),.I(G5AD_P),.IB(G5AD_N));
  
   IBUFDS_DIFF_OUT #(.DIFF_TERM("TRUE"),.IBUF_LOW_PWR("TRUE"),.IOSTANDARD("DEFAULT"))
	IBUFDS_DIFF_OUT_G6AD[15:0] (.O(g6ad),.OB(g6ad_b),.I(G6AD_P),.IB(G6AD_N));
  
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G1ADCLK0 (.O(g1adclk0),.I(G1ADCLK0P),.IB(G1ADCLK0N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G1ADCLK1 (.O(g1adclk1),.I(G1ADCLK1P),.IB(G1ADCLK1N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G1LCLK0 (.O(g1lclk0),.I(G1LCLK0P),.IB(G1LCLK0N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G1LCLK1 (.O(g1lclk1),.I(G1LCLK1P),.IB(G1LCLK1N));
     BUFR #(.BUFR_DIVIDE("BYPASS"),.SIM_DEVICE("VIRTEX6")) BUFR_G1FRM0 (.O(G1FRM_CLK0),.CE(1'b1),.CLR(1'b0),.I(g1adclk0));
     BUFR #(.BUFR_DIVIDE("BYPASS"),.SIM_DEVICE("VIRTEX6")) BUFR_G1FRM1 (.O(G1FRM_CLK1),.CE(1'b1),.CLR(1'b0),.I(g1adclk1));
    BUFIO BUFIO_G1BIT0P (.O(g1bitclk0),.I(g1lclk0));
    BUFIO BUFIO_G1BIT1P (.O(g1bitclk1),.I(g1lclk1));


  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G2ADCLK0 (.O(g2adclk0),.I(G2ADCLK0P),.IB(G2ADCLK0N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G2ADCLK1 (.O(g2adclk1),.I(G2ADCLK1P),.IB(G2ADCLK1N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G2LCLK0 (.O(g2lclk0),.I(G2LCLK0P),.IB(G2LCLK0N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G2LCLK1 (.O(g2lclk1),.I(G2LCLK1P),.IB(G2LCLK1N));
     BUFR #(.BUFR_DIVIDE("BYPASS"),.SIM_DEVICE("VIRTEX6")) BUFR_G2FRM0 (.O(G2FRM_CLK0),.CE(1'b1),.CLR(1'b0),.I(g2adclk0));
     BUFR #(.BUFR_DIVIDE("BYPASS"),.SIM_DEVICE("VIRTEX6")) BUFR_G2FRM1 (.O(G2FRM_CLK1),.CE(1'b1),.CLR(1'b0),.I(g2adclk1));
    BUFIO BUFIO_G2BIT0P (.O(g2bitclk0),.I(g2lclk0));
    BUFIO BUFIO_G2BIT1P (.O(g2bitclk1),.I(g2lclk1));

  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G3ADCLK0 (.O(g3adclk0),.I(G3ADCLK0P),.IB(G3ADCLK0N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G3ADCLK1 (.O(g3adclk1),.I(G3ADCLK1P),.IB(G3ADCLK1N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G3LCLK0 (.O(g3lclk0),.I(G3LCLK0P),.IB(G3LCLK0N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G3LCLK1 (.O(g3lclk1),.I(G3LCLK1P),.IB(G3LCLK1N));
     BUFR #(.BUFR_DIVIDE("BYPASS"),.SIM_DEVICE("VIRTEX6")) BUFR_G3FRM0 (.O(G3FRM_CLK0),.CE(1'b1),.CLR(1'b0),.I(g3adclk0));
     BUFR #(.BUFR_DIVIDE("BYPASS"),.SIM_DEVICE("VIRTEX6")) BUFR_G3FRM1 (.O(G3FRM_CLK1),.CE(1'b1),.CLR(1'b0),.I(g3adclk1));
    BUFIO BUFIO_G3BIT0P (.O(g3bitclk0),.I(g3lclk0));
    BUFIO BUFIO_G3BIT1P (.O(g3bitclk1),.I(g3lclk1));

  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G4ADCLK0 (.O(g4adclk0),.I(G4ADCLK0P),.IB(G4ADCLK0N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G4ADCLK1 (.O(g4adclk1),.I(G4ADCLK1P),.IB(G4ADCLK1N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G4LCLK0 (.O(g4lclk0),.I(G4LCLK0P),.IB(G4LCLK0N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G4LCLK1 (.O(g4lclk1),.I(G4LCLK1P),.IB(G4LCLK1N));
     BUFR #(.BUFR_DIVIDE("BYPASS"),.SIM_DEVICE("VIRTEX6")) BUFR_G4FRM0 (.O(G4FRM_CLK0),.CE(1'b1),.CLR(1'b0),.I(g4adclk0));
     BUFR #(.BUFR_DIVIDE("BYPASS"),.SIM_DEVICE("VIRTEX6")) BUFR_G4FRM1 (.O(G4FRM_CLK1),.CE(1'b1),.CLR(1'b0),.I(g4adclk1));
    BUFIO BUFIO_G4BIT0P (.O(g4bitclk0),.I(g4lclk0));
    BUFIO BUFIO_G4BIT1P (.O(g4bitclk1),.I(g4lclk1));

  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G5ADCLK0 (.O(g5adclk0),.I(G5ADCLK0P),.IB(G5ADCLK0N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G5ADCLK1 (.O(g5adclk1),.I(G5ADCLK1P),.IB(G5ADCLK1N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G5LCLK0 (.O(g5lclk0),.I(G5LCLK0P),.IB(G5LCLK0N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G5LCLK1 (.O(g5lclk1),.I(G5LCLK1P),.IB(G5LCLK1N));
     BUFR #(.BUFR_DIVIDE("BYPASS"),.SIM_DEVICE("VIRTEX6")) BUFR_G5FRM0 (.O(G5FRM_CLK0),.CE(1'b1),.CLR(1'b0),.I(g5adclk0));
     BUFR #(.BUFR_DIVIDE("BYPASS"),.SIM_DEVICE("VIRTEX6")) BUFR_G5FRM1 (.O(G5FRM_CLK1),.CE(1'b1),.CLR(1'b0),.I(g5adclk1));
    BUFIO BUFIO_G5BIT0P (.O(g5bitclk0),.I(g5lclk0));
    BUFIO BUFIO_G5BIT1P (.O(g5bitclk1),.I(g5lclk1));

  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G6ADCLK0 (.O(g6adclk0),.I(G6ADCLK0P),.IB(G6ADCLK0N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G6ADCLK1 (.O(g6adclk1),.I(G6ADCLK1P),.IB(G6ADCLK1N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G6LCLK0 (.O(g6lclk0),.I(G6LCLK0P),.IB(G6LCLK0N));
  IBUFGDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFGDS_G6LCLK1 (.O(g6lclk1),.I(G6LCLK1P),.IB(G6LCLK1N));
     BUFR #(.BUFR_DIVIDE("BYPASS"),.SIM_DEVICE("VIRTEX6")) BUFR_G6FRM0 (.O(G6FRM_CLK0),.CE(1'b1),.CLR(1'b0),.I(g6adclk0));
     BUFR #(.BUFR_DIVIDE("BYPASS"),.SIM_DEVICE("VIRTEX6")) BUFR_G6FRM1 (.O(G6FRM_CLK1),.CE(1'b1),.CLR(1'b0),.I(g6adclk1));
    BUFIO BUFIO_G6BIT0P (.O(g6bitclk0),.I(g6lclk0));
    BUFIO BUFIO_G6BIT1P (.O(g6bitclk1),.I(g6lclk1));

	
//----------------------------------------------------------------------------
// Output     Output      Phase    Duty Cycle   Pk-to-Pk     Phase
// Clock     Freq (MHz)  (degrees)    (%)     Jitter (ps)  Error (ps)
//----------------------------------------------------------------------------
// CLK_OUT1    20.000    270.000      50.0      396.008    301.601             Frame clock
// CLK_OUT2   120.000      0.000      50.0      261.711    301.601             Bit clock odd
// CLK_OUT3   120.000    180.000      50.0      261.711    301.601             Bit clock even
//
//----------------------------------------------------------------------------
// Input Clock   Input Freq (MHz)   Input Jitter (UI)
//----------------------------------------------------------------------------
// primary          20.000            0.010

//  adc_frame_clk g1_adc_frame (.CLK_IN1(g1adclk0),.CLK_IN2(g1adclk1),.CLK_IN_SEL(CLK_SEL[0]),.CLK_OUT1(G1_FRAME_CLOCK),.CLK_OUT2(g1_bit_clk_odd),.CLK_OUT3(g1_bit_clk_even),.RESET(FRM_CLK_RST),.LOCKED(G1_FRM_LOCK));
//  adc_frame_clk g3_adc_frame (.CLK_IN1(g4adclk0),.CLK_IN2(g3adclk1),.CLK_IN_SEL(CLK_SEL[1]),.CLK_OUT1(G3_FRAME_CLOCK),.CLK_OUT2(g3_bit_clk_odd),.CLK_OUT3(g3_bit_clk_even),.RESET(FRM_CLK_RST),.LOCKED(G3_FRM_LOCK));
//  adc_frame_clk g5_adc_frame (.CLK_IN1(g5adclk0),.CLK_IN2(g6adclk1),.CLK_IN_SEL(CLK_SEL[2]),.CLK_OUT1(G5_FRAME_CLOCK),.CLK_OUT2(g5_bit_clk_odd),.CLK_OUT3(g5_bit_clk_even),.RESET(FRM_CLK_RST),.LOCKED(G5_FRM_LOCK));

//assign G5_FRM_LOCK = 1;
//assign G5_FRAME_CLOCK = G3_FRAME_CLOCK;
//assign g5_bit_clk_odd = g3_bit_clk_odd;
//assign g5_bit_clk_even = g3_bit_clk_even;

// Synchronization

   always @(posedge G1FRM_CLK0) begin
		g1bit_slip_odd0 <= bit_slip_odd;
		g1bit_slip_evn0 <= bit_slip_evn;
		g1rst0      <= RST;
	end
   always @(posedge G1FRM_CLK1) begin
		g1bit_slip_odd1 <= bit_slip_odd;
		g1bit_slip_evn1 <= bit_slip_evn;
		g1rst1      <= RST;
	end
   always @(posedge G2FRM_CLK0) begin
		g2bit_slip_odd0 <= bit_slip_odd;
		g2bit_slip_evn0 <= bit_slip_evn;
		g2rst0      <= RST;
	end
   always @(posedge G2FRM_CLK1) begin
		g2bit_slip_odd1 <= bit_slip_odd;
		g2bit_slip_evn1 <= bit_slip_evn;
		g2rst1      <= RST;
	end
   always @(posedge G3FRM_CLK0) begin
		g3bit_slip_odd0 <= bit_slip_odd;
		g3bit_slip_evn0 <= bit_slip_evn;
		g3rst0      <= RST;
	end
   always @(posedge G3FRM_CLK1) begin
		g3bit_slip_odd1 <= bit_slip_odd;
		g3bit_slip_evn1 <= bit_slip_evn;
		g3rst1      <= RST;
	end
   always @(posedge G4FRM_CLK0) begin
		g4bit_slip_odd0 <= bit_slip_odd;
		g4bit_slip_evn0 <= bit_slip_evn;
		g4rst0      <= RST;
	end
   always @(posedge G4FRM_CLK1) begin
		g4bit_slip_odd1 <= bit_slip_odd;
		g4bit_slip_evn1 <= bit_slip_evn;
		g4rst1      <= RST;
	end
   always @(posedge G5FRM_CLK0) begin
		g5bit_slip_odd0 <= bit_slip_odd;
		g5bit_slip_evn0 <= bit_slip_evn;
		g5rst0      <= RST;
	end
   always @(posedge G5FRM_CLK1) begin
		g5bit_slip_odd1 <= bit_slip_odd;
		g5bit_slip_evn1 <= bit_slip_evn;
		g5rst1      <= RST;
	end
   always @(posedge G6FRM_CLK0) begin
		g6bit_slip_odd0 <= bit_slip_odd;
		g6bit_slip_evn0 <= bit_slip_evn;
		g6rst0      <= RST;
	end
   always @(posedge G6FRM_CLK1) begin
		g6bit_slip_odd1 <= bit_slip_odd;
		g6bit_slip_evn1 <= bit_slip_evn;
		g6rst1      <= RST;
	end
	
   genvar ch;
	generate
		for (ch=0; ch<8; ch=ch+1) begin : g1a0
		   deserializer serdes(.BIT_CLK_ODD(~g1bitclk0),.BIT_CLK_EVEN(g1bitclk0),.FRAME_CLK(G1FRM_CLK0),.RST(g1rst0),
										.D(g1ad[ch]),.DDLY(1'b0),.DB(g1ad_b[ch]),.DBDLY(1'b1),.BIT_SLIP_ODD(g1bit_slip_odd0),.BIT_SLIP_EVN(g1bit_slip_evn0),
										.Q(g1dout[ch]),.QODD(g1qodd[ch]),.QEVN(g1qevn[ch]));
		end
	endgenerate
	generate
		for (ch=8; ch<16; ch=ch+1) begin : g1a1
		   deserializer serdes(.BIT_CLK_ODD(~g1bitclk1),.BIT_CLK_EVEN(g1bitclk1),.FRAME_CLK(G1FRM_CLK1),.RST(g1rst1),
										.D(g1ad[ch]),.DDLY(1'b0),.DB(g1ad_b[ch]),.DBDLY(1'b1),.BIT_SLIP_ODD(g1bit_slip_odd1),.BIT_SLIP_EVN(g1bit_slip_evn1),
										.Q(g1dout[ch]),.QODD(g1qodd[ch]),.QEVN(g1qevn[ch]));
		end
	endgenerate
	
	generate
		for (ch=0; ch<8; ch=ch+1) begin : g2a0
		   deserializer serdes(.BIT_CLK_ODD(~g2bitclk0),.BIT_CLK_EVEN(g2bitclk0),.FRAME_CLK(G2FRM_CLK0),.RST(g2rst0),
										.D(g2ad[ch]),.DDLY(1'b0),.DB(g2ad_b[ch]),.DBDLY(1'b1),.BIT_SLIP_ODD(g2bit_slip_odd0),.BIT_SLIP_EVN(g2bit_slip_evn0),
										.Q(g2dout[ch]),.QODD(g2qodd[ch]),.QEVN(g2qevn[ch]));
		end
	endgenerate
	generate
		for (ch=8; ch<16; ch=ch+1) begin : g2a1
		   deserializer serdes(.BIT_CLK_ODD(~g2bitclk1),.BIT_CLK_EVEN(g2bitclk1),.FRAME_CLK(G2FRM_CLK1),.RST(g2rst1),
										.D(g2ad[ch]),.DDLY(1'b0),.DB(g2ad_b[ch]),.DBDLY(1'b1),.BIT_SLIP_ODD(g2bit_slip_odd1),.BIT_SLIP_EVN(g2bit_slip_evn1),
										.Q(g2dout[ch]),.QODD(g2qodd[ch]),.QEVN(g2qevn[ch]));
		end
	endgenerate
	
	generate
		for (ch=0; ch<8; ch=ch+1) begin : g3a0
		   deserializer serdes(.BIT_CLK_ODD(~g3bitclk0),.BIT_CLK_EVEN(g3bitclk0),.FRAME_CLK(G3FRM_CLK0),.RST(g3rst0),
										.D(g3ad[ch]),.DDLY(1'b0),.DB(g3ad_b[ch]),.DBDLY(1'b1),.BIT_SLIP_ODD(g3bit_slip_odd0),.BIT_SLIP_EVN(g3bit_slip_evn0),
										.Q(g3dout[ch]),.QODD(g3qodd[ch]),.QEVN(g3qevn[ch]));
		end
	endgenerate
	generate
		for (ch=8; ch<16; ch=ch+1) begin : g3a1
		   deserializer serdes(.BIT_CLK_ODD(~g3bitclk1),.BIT_CLK_EVEN(g3bitclk1),.FRAME_CLK(G3FRM_CLK1),.RST(g3rst1),
										.D(g3ad[ch]),.DDLY(1'b0),.DB(g3ad_b[ch]),.DBDLY(1'b1),.BIT_SLIP_ODD(g3bit_slip_odd1),.BIT_SLIP_EVN(g3bit_slip_evn1),
										.Q(g3dout[ch]),.QODD(g3qodd[ch]),.QEVN(g3qevn[ch]));
		end
	endgenerate
	
	generate
		for (ch=0; ch<8; ch=ch+1) begin : g4a0
		   deserializer serdes(.BIT_CLK_ODD(~g4bitclk0),.BIT_CLK_EVEN(g4bitclk0),.FRAME_CLK(G4FRM_CLK0),.RST(g4rst0),
										.D(g4ad[ch]),.DDLY(1'b0),.DB(g4ad_b[ch]),.DBDLY(1'b1),.BIT_SLIP_ODD(g4bit_slip_odd0),.BIT_SLIP_EVN(g4bit_slip_evn0),
										.Q(g4dout[ch]),.QODD(g4qodd[ch]),.QEVN(g4qevn[ch]));
		end
	endgenerate
	generate
		for (ch=8; ch<16; ch=ch+1) begin : g4a1
		   deserializer serdes(.BIT_CLK_ODD(~g4bitclk1),.BIT_CLK_EVEN(g4bitclk1),.FRAME_CLK(G4FRM_CLK1),.RST(g4rst1),
										.D(g4ad[ch]),.DDLY(1'b0),.DB(g4ad_b[ch]),.DBDLY(1'b1),.BIT_SLIP_ODD(g4bit_slip_odd1),.BIT_SLIP_EVN(g4bit_slip_evn1),
										.Q(g4dout[ch]),.QODD(g4qodd[ch]),.QEVN(g4qevn[ch]));
		end
	endgenerate
	
	generate
		for (ch=0; ch<8; ch=ch+1) begin : g5a0
		   deserializer serdes(.BIT_CLK_ODD(~g5bitclk0),.BIT_CLK_EVEN(g5bitclk0),.FRAME_CLK(G5FRM_CLK0),.RST(g5rst0),
										.D(g5ad[ch]),.DDLY(1'b0),.DB(g5ad_b[ch]),.DBDLY(1'b1),.BIT_SLIP_ODD(g5bit_slip_odd0),.BIT_SLIP_EVN(g5bit_slip_evn0),
										.Q(g5dout[ch]),.QODD(g5qodd[ch]),.QEVN(g5qevn[ch]));
		end
	endgenerate
	generate
		for (ch=8; ch<16; ch=ch+1) begin : g5a1
		   deserializer serdes(.BIT_CLK_ODD(~g5bitclk1),.BIT_CLK_EVEN(g5bitclk1),.FRAME_CLK(G5FRM_CLK1),.RST(g5rst1),
										.D(g5ad[ch]),.DDLY(1'b0),.DB(g5ad_b[ch]),.DBDLY(1'b1),.BIT_SLIP_ODD(g5bit_slip_odd1),.BIT_SLIP_EVN(g5bit_slip_evn1),
										.Q(g5dout[ch]),.QODD(g5qodd[ch]),.QEVN(g5qevn[ch]));
		end
	endgenerate
	
	generate
		for (ch=0; ch<8; ch=ch+1) begin : g6a0
		   deserializer serdes(.BIT_CLK_ODD(~g6bitclk0),.BIT_CLK_EVEN(g6bitclk0),.FRAME_CLK(G6FRM_CLK0),.RST(g6rst0),
										.D(g6ad[ch]),.DDLY(1'b0),.DB(g6ad_b[ch]),.DBDLY(1'b1),.BIT_SLIP_ODD(g6bit_slip_odd0),.BIT_SLIP_EVN(g6bit_slip_evn0),
										.Q(g6dout[ch]),.QODD(g6qodd[ch]),.QEVN(g6qevn[ch]));
		end
	endgenerate
	generate
		for (ch=8; ch<16; ch=ch+1) begin : g6a1
		   deserializer serdes(.BIT_CLK_ODD(~g6bitclk1),.BIT_CLK_EVEN(g6bitclk1),.FRAME_CLK(G6FRM_CLK1),.RST(g6rst1),
										.D(g6ad[ch]),.DDLY(1'b0),.DB(g6ad_b[ch]),.DBDLY(1'b1),.BIT_SLIP_ODD(g6bit_slip_odd1),.BIT_SLIP_EVN(g6bit_slip_evn1),
										.Q(g6dout[ch]),.QODD(g6qodd[ch]),.QEVN(g6qevn[ch]));
		end
	endgenerate
	
always @(posedge CLK20 or posedge wrst) begin
   if(wrst)
	   wait_count <= 3'h0;
	else
	   if(winc)
		   wait_count <= wait_count +1;
		else
		   wait_count <= wait_count;
end

always @(posedge CLK20 or posedge RST) begin
   if(RST)
	   evn_count <= 4'h0;
	else
	   if(bit_slip_evn)
		   evn_count <= evn_count +1;
		else
		   evn_count <= evn_count;
end

always @(posedge CLK20 or posedge RST) begin
   if(RST)
	   odd_count <= 4'h0;
	else
	   if(bit_slip_odd)
		   odd_count <= odd_count +1;
		else
		   odd_count <= odd_count;
end

   DSR_align 
	DSR_align_FSM1(
     .ALIGNED(DSR_ALIGNED),
     .BIT_SLIP_EVN(bit_slip_evn),
     .BIT_SLIP_ODD(bit_slip_odd),
     .WINC(winc),
     .WRST(wrst),
     .CLK(CLK20),
     .EVNCNT(evn_count),
     .ODDCNT(odd_count),
     .RST(RST),
     .WCNT(wait_count) 
);
	
endmodule


// Created by fizzim.pl version 3.60 on 2011:05:03 at 14:02:14 (www.fizzim.com)

module DSR_align (
  output reg ALIGNED,
  output reg BIT_SLIP_EVN,
  output reg BIT_SLIP_ODD,
  output reg WINC,
  output reg WRST,
  input CLK,
  input wire [3:0] EVNCNT,
  input wire [3:0] ODDCNT,
  input RST,
  input wire [2:0] WCNT 
);
  
  // state bits
  parameter 
  Start      = 3'b000, 
  Aligned    = 3'b001, 
  BS_even    = 3'b010, 
  BS_odd     = 3'b011, 
  BSevn_Wait = 3'b100, 
  BSodd_Wait = 3'b101, 
  Wait1      = 3'b110; 
  
  reg [2:0] state;
  reg [2:0] nextstate;
  
  // comb always block
  always @* begin
    nextstate = 3'bx; // default to x because default_state_is_x is set
    ALIGNED = 0; // default
    BIT_SLIP_EVN = 0; // default
    BIT_SLIP_ODD = 0; // default
    WINC = 0; // default
    WRST = 0; // default
    case (state)
      Start     : begin
                                                WRST = 1;
                                                nextstate = Wait1;
      end
      Aligned   : begin
                                                ALIGNED = 1;
                                                nextstate = Aligned;
      end
      BS_even   : begin
                                                BIT_SLIP_EVN = 1;
                                                WRST = 1;
                                                nextstate = BSevn_Wait;
      end
      BS_odd    : begin
                                                BIT_SLIP_ODD = 1;
                                                WRST = 1;
                                                nextstate = BSodd_Wait;
      end
      BSevn_Wait: begin
                                                WINC = 1;
        if      ((WCNT == 4) && (EVNCNT == 4))  nextstate = BS_odd;
        else if ((WCNT == 4))                   nextstate = BS_even;
        else                                    nextstate = BSevn_Wait;
      end
      BSodd_Wait: begin
                                                WINC = 1;
        if      ((WCNT == 4) && (ODDCNT == 5))  nextstate = Aligned;
        else if ((WCNT == 4))                   nextstate = BS_odd;
        else                                    nextstate = BSodd_Wait;
      end
      Wait1     : begin
                                                WINC = 1;
        if      (WCNT == 5)                     nextstate = BS_even;
        else                                    nextstate = Wait1;
      end
    endcase
  end
  
  // Assign reg'd outputs to state bits
  
  // sequential always block
  always @(posedge CLK or posedge RST) begin
    if (RST)
      state <= Start;
    else
      state <= nextstate;
  end
  
  // This code allows you to see state names in simulation
  `ifndef SYNTHESIS
  reg [79:0] statename;
  always @* begin
    case (state)
      Start     : statename = "Start";
      Aligned   : statename = "Aligned";
      BS_even   : statename = "BS_even";
      BS_odd    : statename = "BS_odd";
      BSevn_Wait: statename = "BSevn_Wait";
      BSodd_Wait: statename = "BSodd_Wait";
      Wait1     : statename = "Wait1";
      default   : statename = "XXXXXXXXXX";
    endcase
  end
  `endif

endmodule

