//-----------------------------------------------------------------------------
// Title      : Virtex-6 Embedded Tri-Mode Ethernet MAC Wrapper Example Design
// Project    : Virtex-6 Embedded Tri-Mode Ethernet MAC Wrapper
// File       : DAQ_MAC_example_design.v
// Version    : 1.4
//-----------------------------------------------------------------------------
//
// (c) Copyright 2009-2010 Xilinx, Inc. All rights reserved.
//
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
//
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
//
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
//
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.
//
//-----------------------------------------------------------------------------
// Description:  This is the Example Design wrapper for the Virtex-6
//               Embedded Tri-Mode Ethernet MAC. It is intended that this
//               example design can be quickly adapted and downloaded onto an
//               FPGA to provide a hardware test environment.
//
//               The Example Design wrapper:
//
//               * instantiates the EMAC LocalLink-level wrapper (the EMAC
//                 block-level wrapper with the RX and TX FIFOs and a
//                 LocalLink interface);
//
//               * instantiates a simple example design which provides an
//                 address swap and loopback function at the user interface;
//
//               * instantiates the fundamental clocking resources required
//                 by the core;
//
//               Please refer to the Datasheet, Getting Started Guide, and
//               the Virtex-6 Embedded Tri-Mode Ethernet MAC User Gude for
//               further information.
//
//    ---------------------------------------------------------------------
//    |EXAMPLE DESIGN WRAPPER                                             |
//    |           --------------------------------------------------------|
//    |           |LOCALLINK-LEVEL WRAPPER                                |
//    |           |              -----------------------------------------|
//    |           |              |BLOCK-LEVEL WRAPPER                     |
//    |           |              |    ---------------------               |
//    | --------  |  ----------  |    | INSTANCE-LEVEL    |               |
//    | |      |  |  |        |  |    | WRAPPER           |  ---------    |
//    | |      |->|->|        |->|--->| Tx            Tx  |->|       |--->|
//    | |      |  |  |        |  |    | client        PHY |  |       |    |
//    | | ADDR |  |  | LOCAL- |  |    | I/F           I/F |  |       |    |
//    | | SWAP |  |  | LINK   |  |    |                   |  | PHY   |    |
//    | |      |  |  | FIFO   |  |    |                   |  | I/F   |    |
//    | |      |  |  |        |  |    |                   |  |       |    |
//    | |      |  |  |        |  |    | Rx            Rx  |  |       |    |
//    | |      |  |  |        |  |    | client        PHY |  |       |    |
//    | |      |<-|<-|        |<-|<---| I/F           I/F |<-|       |<---|
//    | |      |  |  |        |  |    |                   |  ---------    |
//    | --------  |  ----------  |    ---------------------               |
//    |           |              -----------------------------------------|
//    |           --------------------------------------------------------|
//    ---------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

`timescale 1 ps / 1 ps


//-----------------------------------------------------------------------------
// Module declaration for the example design
//-----------------------------------------------------------------------------

module DAQ_MAC_example_design
(

    // Client receiver interface
    EMACCLIENTRXDVLD,
    EMACCLIENTRXFRAMEDROP,
    EMACCLIENTRXSTATS,
    EMACCLIENTRXSTATSVLD,
    EMACCLIENTRXSTATSBYTEVLD,

    // Client transmitter interface
    CLIENTEMACTXIFGDELAY,
    EMACCLIENTTXSTATS,
    EMACCLIENTTXSTATSVLD,
    EMACCLIENTTXSTATSBYTEVLD,

    // MAC control interface
    CLIENTEMACPAUSEREQ,
    CLIENTEMACPAUSEVAL,

    // EMAC-transceiver link status
    EMACCLIENTSYNCACQSTATUS,
    EMACANINTERRUPT,

    // 1000BASE-X PCS/PMA interface
    TXP,
    TXN,
    RXP,
    RXN,
    PHYAD,

    // 1000BASE-X PCS/PMA clock buffer input
    MGTCLK_N,
    MGTCLK_P,

    // Asynchronous reset
    RESET
);


//-----------------------------------------------------------------------------
// Port declarations
//-----------------------------------------------------------------------------

    // Client receiver interface
    output          EMACCLIENTRXDVLD;
    output          EMACCLIENTRXFRAMEDROP;
    output   [6:0]  EMACCLIENTRXSTATS;
    output          EMACCLIENTRXSTATSVLD;
    output          EMACCLIENTRXSTATSBYTEVLD;

    // Client transmitter interface
    input    [7:0]  CLIENTEMACTXIFGDELAY;
    output          EMACCLIENTTXSTATS;
    output          EMACCLIENTTXSTATSVLD;
    output          EMACCLIENTTXSTATSBYTEVLD;

    // MAC control interface
    input           CLIENTEMACPAUSEREQ;
    input   [15:0]  CLIENTEMACPAUSEVAL;

    //EMAC-transceiver link status
    output          EMACCLIENTSYNCACQSTATUS;
    output          EMACANINTERRUPT;

    // 1000BASE-X PCS/PMA interface
    output          TXP;
    output          TXN;
    input           RXP;
    input           RXN;
    input    [4:0]  PHYAD;

    // 1000BASE-X PCS/PMA clock buffer input
    input           MGTCLK_N;
    input           MGTCLK_P;

    // Asynchronous reset
    input           RESET;


//-----------------------------------------------------------------------------
// Wire and register declarations
//-----------------------------------------------------------------------------

    // Global asynchronous reset
    wire            reset_i;

    // LocalLink interface clocking signal
    wire            ll_clk_i;

    // Address swap transmitter connections
    wire     [15:0] tx_ll_data_i;
    wire            tx_ll_rem_i;
    wire            tx_ll_sof_n_i;
    wire            tx_ll_eof_n_i;
    wire            tx_ll_src_rdy_n_i;
    wire            tx_ll_dst_rdy_n_i;

    // Address swap receiver connections
    wire     [15:0] rx_ll_data_i;
    wire            rx_ll_rem_i;
    wire            rx_ll_sof_n_i;
    wire            rx_ll_eof_n_i;
    wire            rx_ll_src_rdy_n_i;
    wire            rx_ll_dst_rdy_n_i;

    // Synchronous reset registers in the LocalLink clock domain
    (* ASYNC_REG = "TRUE" *)
    reg       [5:0] ll_pre_reset_i;

    reg             ll_reset_i;

    // Reset signal from the transceiver
    wire            resetdone_i;
    (* ASYNC_REG = "TRUE" *)
    reg             resetdone_r;

    // Transceiver output clock (REFCLKOUT at 125MHz)
    wire            clk125_o;

    // 125MHz clock input to wrappers
    (* KEEP = "TRUE" *)
    wire            clk125;

    // Input 125MHz differential clock for transceiver
    wire            clk_ds;

    // Buffered output clock from transceiver for MMCM
    wire            clk125_o_bufg;
    // Clock signals for MMCM at 2x GT reference rate
    wire            clkfb;
    wire            clkfb_pre_bufg;
    wire            clk125_pre_bufg;
    wire            clk2x_pre_bufg;
    wire            client_mmcm_locked_i;
    reg             client_mmcm_locked_r;
    reg             client_mmcm_locked_r2;

    // 2x clock to wrappers
    wire            clk2x;


//-----------------------------------------------------------------------------
// Main body of code
//-----------------------------------------------------------------------------

    // Reset input buffer
    IBUF reset_ibuf (
       .I (RESET),
       .O (reset_i)
    );

    // Generate the clock input to the transceiver
    // (clk_ds can be shared between multiple EMAC instances, including
    //  multiple instantiations of the EMAC wrappers)
    IBUFDS_GTXE1 clkingen (
      .I     (MGTCLK_P),
      .IB    (MGTCLK_N),
      .CEB   (1'b0),
      .O     (clk_ds),
      .ODIV2 ()
    );

    // The 125MHz clock from the transceiver is routed through a BUFG and
    // input to the MMCM
    BUFG bufg_clk125 (
       .I (clk125_o),
       .O (clk125_o_bufg)
    );

    // ** The wrappers have been configured to use a 16-bit client interface
    //    while still operating the EMAC at a rate of 1 Gb/s. Please note
    //    that, in this configuration, the clock "clk125" runs at 62.5 MHz
    //    and is constrained as such in the provided UCF file. **
    //
    // Use an MMCM to divide the 125MHz clock from the transceiver by 2
    // to get a 62.5MHz clock for EMAC.
    MMCM_BASE #(
        .CLKFBOUT_MULT_F  (5.0),
        .CLKOUT0_DIVIDE_F (5.0),
        .CLKOUT1_DIVIDE   (10),
        .CLKIN1_PERIOD    (8.0),
        .REF_JITTER1      (0.010)
    )
    client_mmcm (
        .CLKFBOUT  (clkfb_pre_bufg),
        .CLKFBOUTB (),
        .CLKOUT0   (clk2x_pre_bufg),
        .CLKOUT0B  (),
        .CLKOUT1   (clk125_pre_bufg),
        .CLKOUT1B  (),
        .CLKOUT2   (),
        .CLKOUT2B  (),
        .CLKOUT3   (),
        .CLKOUT3B  (),
        .CLKOUT4   (),
        .CLKOUT5   (),
        .CLKOUT6   (),
        .LOCKED    (client_mmcm_locked_i),
        .CLKFBIN   (clkfb),
        .CLKIN1    (clk125_o_bufg),
        .PWRDWN    (1'b0),
        .RST       (reset_i)
    );

    // Globally-buffer the feedback clock from the MMCM
    BUFG feedback_clk_bufg (
       .I (clkfb_pre_bufg),
       .O (clkfb)
    );

    // Globally-buffer the 2x clock from the MMCM
    // (clk2x can be shared between multiple EMAC instances, including
    //  multiple instantiations of the EMAC wrappers)
    BUFG tx_client_clk_out_bufg (
       .I (clk2x_pre_bufg),
       .O (clk2x)
    );

    // The 125MHz clock from the transceiver (via the MMCM) is routed
    // through a BUFG and input to the MAC wrappers
    // (clk125 can be shared between multiple EMAC instances, including
    //  multiple instantiations of the EMAC wrappers)
    BUFG tx_client_clk_in_div2_bufg (
       .I (clk125_pre_bufg),
       .O (clk125)
    );

    // Register the MMCM locked indicator to assist with timing closure
    // and ensure clock stability on bring-up
    always @(posedge clk125, posedge reset_i)
    if (reset_i == 1'b1)
    begin
       client_mmcm_locked_r  <= 1'b0;
       client_mmcm_locked_r2 <= 1'b0;
    end
    else
    begin
       client_mmcm_locked_r  <= client_mmcm_locked_i;
       client_mmcm_locked_r2 <= client_mmcm_locked_r;
    end

    // Clock the LocalLink interface with the phase-aligned and
    // globally-buffered 125MHz clock from the MMCM
    assign ll_clk_i = clk125;

    //------------------------------------------------------------------------
    // Instantiate the LocalLink-level EMAC wrapper (DAQ_MAC_locallink.v)
    //------------------------------------------------------------------------
    DAQ_MAC_locallink DAQ_MAC_locallink_inst
    (
    // 125MHz clock output from transceiver
    .CLK125_OUT               (clk125_o),
    // 125MHz clock input from BUFG
    .CLK125                   (clk125),
    // 2x clock input from MMCM
    .CLK2X                    (clk2x),
    .CLK2X_MMCM_LOCKED        (client_mmcm_locked_r2),

    // LocalLink receiver interface
    .RX_LL_CLOCK              (ll_clk_i),
    .RX_LL_RESET              (ll_reset_i),
    .RX_LL_DATA               (rx_ll_data_i),
    .RX_LL_SOF_N              (rx_ll_sof_n_i),
    .RX_LL_EOF_N              (rx_ll_eof_n_i),
    .RX_LL_SRC_RDY_N          (rx_ll_src_rdy_n_i),
    .RX_LL_DST_RDY_N          (rx_ll_dst_rdy_n_i),
    .RX_LL_REM                (rx_ll_rem_i),
    .RX_LL_FIFO_STATUS        (),

    // Client receiver signals
    .EMACCLIENTRXDVLD         (EMACCLIENTRXDVLD),
    .EMACCLIENTRXFRAMEDROP    (EMACCLIENTRXFRAMEDROP),
    .EMACCLIENTRXSTATS        (EMACCLIENTRXSTATS),
    .EMACCLIENTRXSTATSVLD     (EMACCLIENTRXSTATSVLD),
    .EMACCLIENTRXSTATSBYTEVLD (EMACCLIENTRXSTATSBYTEVLD),

    // LocalLink transmitter interface
    .TX_LL_CLOCK              (ll_clk_i),
    .TX_LL_RESET              (ll_reset_i),
    .TX_LL_DATA               (tx_ll_data_i),
    .TX_LL_SOF_N              (tx_ll_sof_n_i),
    .TX_LL_EOF_N              (tx_ll_eof_n_i),
    .TX_LL_SRC_RDY_N          (tx_ll_src_rdy_n_i),
    .TX_LL_DST_RDY_N          (tx_ll_dst_rdy_n_i),
    .TX_LL_REM                (tx_ll_rem_i),

    // Client transmitter signals
    .CLIENTEMACTXIFGDELAY     (CLIENTEMACTXIFGDELAY),
    .EMACCLIENTTXSTATS        (EMACCLIENTTXSTATS),
    .EMACCLIENTTXSTATSVLD     (EMACCLIENTTXSTATSVLD),
    .EMACCLIENTTXSTATSBYTEVLD (EMACCLIENTTXSTATSBYTEVLD),

    // MAC control interface
    .CLIENTEMACPAUSEREQ       (CLIENTEMACPAUSEREQ),
    .CLIENTEMACPAUSEVAL       (CLIENTEMACPAUSEVAL),

    // EMAC-transceiver link status
    .EMACCLIENTSYNCACQSTATUS  (EMACCLIENTSYNCACQSTATUS),
    .EMACANINTERRUPT          (EMACANINTERRUPT),

    // 1000BASE-X PCS/PMA interface
    .TXP                      (TXP),
    .TXN                      (TXN),
    .RXP                      (RXP),
    .RXN                      (RXN),
    .PHYAD                    (PHYAD),
    .RESETDONE                (resetdone_i),

    // 1000BASE-X PCS/PMA clock buffer input
    .CLK_DS                   (clk_ds),

    // Asynchronous reset
    .RESET                    (reset_i)
    );

    //-------------------------------------------------------------------
    //  Instatiate the address swapping module
    //-------------------------------------------------------------------
    address_swap_module_16 client_side_asm (
       .rx_ll_clock         (ll_clk_i),
       .rx_ll_reset         (ll_reset_i),
       .rx_ll_data_in       (rx_ll_data_i),
       .rx_ll_sof_in_n      (rx_ll_sof_n_i),
       .rx_ll_eof_in_n      (rx_ll_eof_n_i),
       .rx_ll_src_rdy_in_n  (rx_ll_src_rdy_n_i),
       .rx_ll_rem_in        (rx_ll_rem_i),
       .rx_ll_data_out      (tx_ll_data_i),
       .rx_ll_sof_out_n     (tx_ll_sof_n_i),
       .rx_ll_eof_out_n     (tx_ll_eof_n_i),
       .rx_ll_src_rdy_out_n (tx_ll_src_rdy_n_i),
       .rx_ll_rem_out       (tx_ll_rem_i),
       .rx_ll_dst_rdy_in_n  (tx_ll_dst_rdy_n_i)
    );

    assign rx_ll_dst_rdy_n_i = tx_ll_dst_rdy_n_i;

    // Synchronize resetdone_i from the GT in the transmitter clock domain
    always @(posedge ll_clk_i, posedge reset_i)
      if (reset_i === 1'b1)
        resetdone_r <= 1'b0;
      else
        resetdone_r <= resetdone_i;

    // Create synchronous reset in the transmitter clock domain
    always @(posedge ll_clk_i, posedge reset_i)
    begin
      if (reset_i === 1'b1)
      begin
        ll_pre_reset_i <= 6'h3F;
        ll_reset_i     <= 1'b1;
      end
      else if (resetdone_r === 1'b1)
      begin
        ll_pre_reset_i[0]   <= 1'b0;
        ll_pre_reset_i[5:1] <= ll_pre_reset_i[4:0];
        ll_reset_i          <= ll_pre_reset_i[5];
      end
    end


endmodule
