------------------------------------------------------------------------
-- Title      : Demo Testbench
-- Project    : Virtex-6 Embedded Tri-Mode Ethernet MAC Wrapper
-- File       : demo_tb.vhd
-- Version    : 1.4
-------------------------------------------------------------------------------
--
-- (c) Copyright 2009-2010 Xilinx, Inc. All rights reserved.
--
-- This file contains confidential and proprietary information
-- of Xilinx, Inc. and is protected under U.S. and
-- international copyright and other intellectual property
-- laws.
--
-- DISCLAIMER
-- This disclaimer is not a license and does not grant any
-- rights to the materials distributed herewith. Except as
-- otherwise provided in a valid license issued to you by
-- Xilinx, and to the maximum extent permitted by applicable
-- law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
-- WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
-- AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
-- BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
-- INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
-- (2) Xilinx shall not be liable (whether in contract or tort,
-- including negligence, or under any other theory of
-- liability) for any loss or damage of any kind or nature
-- related to, arising under or in connection with these
-- materials, including for any direct, or any indirect,
-- special, incidental, or consequential loss or damage
-- (including loss of data, profits, goodwill, or any type of
-- loss or damage suffered as a result of any action brought
-- by a third party) even if such damage or loss was
-- reasonably foreseeable or Xilinx had been advised of the
-- possibility of the same.
--
-- CRITICAL APPLICATIONS
-- Xilinx products are not designed or intended to be fail-
-- safe, or for use in any application requiring fail-safe
-- performance, such as life-support or safety devices or
-- systems, Class III medical devices, nuclear facilities,
-- applications related to the deployment of airbags, or any
-- other applications that could lead to death, personal
-- injury, or severe property or environmental damage
-- (individually and collectively, "Critical
-- Applications"). Customer assumes the sole risk and
-- liability of any use of Xilinx products in Critical
-- Applications, subject only to applicable laws and
-- regulations governing limitations on product liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
-- PART OF THIS FILE AT ALL TIMES.
--
------------------------------------------------------------------------
-- Description: This testbench will exercise the PHY ports of the EMAC
--              to demonstrate the functionality.
------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity testbench is
end testbench;


architecture behavioral of testbench is


  ----------------------------------------------------------------------
  -- Component declaration for DAQ_MAC_example_design
  --                           (the top level EMAC example deisgn)
  ----------------------------------------------------------------------
  component DAQ_MAC_example_design
    port(
        -- Client receiver interface
        EMACCLIENTRXDVLD         : out std_logic;
        EMACCLIENTRXFRAMEDROP    : out std_logic;
        EMACCLIENTRXSTATS        : out std_logic_vector(6 downto 0);
        EMACCLIENTRXSTATSVLD     : out std_logic;
        EMACCLIENTRXSTATSBYTEVLD : out std_logic;

        -- Client transmitter interface
        CLIENTEMACTXIFGDELAY     : in  std_logic_vector(7 downto 0);
        EMACCLIENTTXSTATS        : out std_logic;
        EMACCLIENTTXSTATSVLD     : out std_logic;
        EMACCLIENTTXSTATSBYTEVLD : out std_logic;

        -- MAC Control interface
        CLIENTEMACPAUSEREQ       : in  std_logic;
        CLIENTEMACPAUSEVAL       : in  std_logic_vector(15 downto 0);

        -- 1000BASE-X PCS/PMA interface
        TXP                      : out std_logic;
        TXN                      : out std_logic;
        RXP                      : in  std_logic;
        RXN                      : in  std_logic;
        PHYAD                    : in  std_logic_vector(4 downto 0);

        EMACCLIENTSYNCACQSTATUS  : out std_logic;
        EMACANINTERRUPT          : out std_logic;

        -- Transceiver differential reference clock
        MGTCLK_P                 : in  std_logic;
        MGTCLK_N                 : in  std_logic;

        -- Asynchronous Reset
        RESET                    : in  std_logic
      );
  end component;


  component configuration_tb is
    port(
      reset                 : out std_logic;

      ------------------------------------------------------------------
      -- Host interface: host_clk is always required
      ------------------------------------------------------------------
      host_clk              : out std_logic;

      ------------------------------------------------------------------
      -- Testbench semaphores
      ------------------------------------------------------------------
      sync_acq_status       : in  std_logic;
      configuration_busy    : out boolean;
      monitor_finished_1g   : in  boolean;
      monitor_finished_100m : in  boolean;
      monitor_finished_10m  : in  boolean
    );
  end component;


  ----------------------------------------------------------------------
  -- Component declaration for the PHY stimulus and monitor
  ----------------------------------------------------------------------

  component phy_tb is
    port(
      clk125m               : in std_logic;

      ------------------------------------------------------------------
      -- GMII interface
      ------------------------------------------------------------------
      txp                   : in  std_logic;
      txn                   : in  std_logic;
      rxp                   : out std_logic;
      rxn                   : out std_logic;

      ------------------------------------------------------------------
      -- Testbench semaphores
      ------------------------------------------------------------------
      configuration_busy    : in  boolean;
      monitor_finished_1g   : out boolean;
      monitor_finished_100m : out boolean;
      monitor_finished_10m  : out boolean
    );
  end component;


  ----------------------------------------------------------------------
  -- Testbench signals
  ----------------------------------------------------------------------
    signal reset              : std_logic := '1';

    signal tx_client_clk      : std_logic;
    signal tx_ifg_delay       : std_logic_vector(7 downto 0)  := (others => '0');
    signal rx_client_clk      : std_logic;
    signal pause_val          : std_logic_vector(15 downto 0) := (others => '0');
    signal pause_req          : std_logic := '0';

    -- 1000BASE-X PCS/PMA signals
    signal txp                : std_logic;
    signal txn                : std_logic;
    signal rxp                : std_logic;
    signal rxn                : std_logic;
    signal sync_acq_status    : std_logic;
    signal phyad              : std_logic_vector(4 downto 0);

    -- Clock signals
    signal host_clk           : std_logic := '0';
    signal gtx_clk            : std_logic;
    signal mgtclk_p           : std_logic := '0';
    signal mgtclk_n           : std_logic := '1';

    ------------------------------------------------------------------
    -- Testbench semaphores
    ------------------------------------------------------------------
    signal configuration_busy    : boolean := false;
    signal monitor_finished_1g   : boolean := false;
    signal monitor_finished_100m : boolean := false;
    signal monitor_finished_10m  : boolean := false;


begin

  ----------------------------------------------------------------------
  -- Wire up device under test
  ----------------------------------------------------------------------
  dut : DAQ_MAC_example_design
  port map (
    -- Client receiver interface
    EMACCLIENTRXDVLD               => open,
    EMACCLIENTRXFRAMEDROP          => open,
    EMACCLIENTRXSTATS              => open,
    EMACCLIENTRXSTATSVLD           => open,
    EMACCLIENTRXSTATSBYTEVLD       => open,

    -- Client transmitter interface
    CLIENTEMACTXIFGDELAY           => tx_ifg_delay,
    EMACCLIENTTXSTATS              => open,
    EMACCLIENTTXSTATSVLD           => open,
    EMACCLIENTTXSTATSBYTEVLD       => open,

    -- MAC Control interface
    CLIENTEMACPAUSEREQ             => pause_req,
    CLIENTEMACPAUSEVAL             => pause_val,

    EMACCLIENTSYNCACQSTATUS        => sync_acq_status,
    EMACANINTERRUPT                => open,

    -- 1000BASE-X PCS/PMA interface
    TXP                           => txp,
    TXN                           => txn,
    RXP                           => rxp,
    RXN                           => rxn,
    PHYAD                         => phyad,

    -- Transceiver differential reference clock
    MGTCLK_P                        =>  mgtclk_p,
    MGTCLK_N                        =>  mgtclk_n,

    -- Asynchronous Reset
    RESET                           => reset
  );


    ----------------------------------------------------------------------------
    -- Flow control is unused in this demonstration
    ----------------------------------------------------------------------------
    pause_req <= '0';
    pause_val <= "0000000000000000";

    -- Tie-off PHY address to a default value
    phyad <= "00001";


    ----------------------------------------------------------------------------
    -- Clock drivers
    ----------------------------------------------------------------------------

    -- Drive GTX_CLK at 125 MHz
    p_gtx_clk : process
    begin
        gtx_clk <= '0';
        wait for 10 ns;
        loop
            wait for 4 ns;
            gtx_clk <= '1';
            wait for 4 ns;
            gtx_clk <= '0';
        end loop;
    end process p_gtx_clk;

    -- Drive serial transceiver differential clock at 125MHz
    mgtclk_p <= gtx_clk;
    mgtclk_n <= not gtx_clk;


  ----------------------------------------------------------------------
  -- Instantiate the PHY stimulus and monitor
  ----------------------------------------------------------------------

  phy_test: phy_tb
    port map (
      clk125m               => gtx_clk,
      ------------------------------------------------------------------
      -- GMII interface
      ------------------------------------------------------------------
      txp                   => txp,
      txn                   => txn,
      rxp                   => rxp,
      rxn                   => rxn,
      
      ------------------------------------------------------------------
      -- Testbench semaphores
      ------------------------------------------------------------------
      configuration_busy    => configuration_busy,
      monitor_finished_1g   => monitor_finished_1g,
      monitor_finished_100m => monitor_finished_100m,
      monitor_finished_10m  => monitor_finished_10m
    );

  ----------------------------------------------------------------------
  -- Instantiate the no-host configuration stimulus
  ----------------------------------------------------------------------

  config_test: configuration_tb
    port map (
      reset                 => reset,

      ------------------------------------------------------------------
      -- Host interface: host_clk is always required
      ------------------------------------------------------------------
      host_clk              => host_clk,

      sync_acq_status       => sync_acq_status,
      configuration_busy    => configuration_busy,
      monitor_finished_1g   => monitor_finished_1g,
      monitor_finished_100m => monitor_finished_100m,
      monitor_finished_10m  => monitor_finished_10m
    );

end behavioral;
