`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:07:59 04/01/2011 
// Design Name: 
// Module Name:    trigger 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module trigger(
	input CLK40,
	// external connections
	input SKW_L1A_P,
	input SKW_L1A_N,
	input SKW_LCT_P,
	input SKW_LCT_N,
	input SKW_RESYNC_P,
	input SKW_RESYNC_N,
	input FMU_L1A,
	input FMU_RESYNC,
	input FMU_L1A_MATCH,
	// internal signals
	input FEM_L1A,
	input FEM_RESYNC,
	input FEM_L1A_MATCH,
	input [1:0] TTC_SRC,
	// common signals
	output reg L1A,
	output reg L1A_MATCH,
	output reg LCT,
	output reg RESYNC
	);
  // TTC configuration modes
  parameter 
  FF_EMU_mode    = 2'b00, 
  FF_FEM_mode    = 2'b01, 
  Skew_Clr_mode  = 2'b10;

wire skw_rw_l1a;
wire skw_rw_lct;
wire skw_rw_resync;
wire fmu_rw_l1a;
wire fmu_rw_l1a_match;
wire fmu_rw_resync;

  IBUFDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFDS_SKW_L1A (.O(skw_rw_l1a),.I(SKW_L1A_P),.IB(SKW_L1A_N));
  IBUFDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFDS_SKW_LCT (.O(skw_rw_lct),.I(SKW_LCT_P),.IB(SKW_LCT_N));
  IBUFDS #(.DIFF_TERM("TRUE"),.IOSTANDARD("DEFAULT")) IBUFDS_SKW_RESYNC (.O(skw_rw_resync),.I(SKW_RESYNC_P),.IB(SKW_RESYNC_N));
  IBUF IBUF_FMU_L1A (.O(fmu_rw_l1a),.I(FMU_L1A));
  IBUF IBUF_FMU_RESYNC (.O(fmu_rw_resync),.I(FMU_RESYNC));
  IBUF IBUF_FMU_EXP (.O(fmu_rw_l1a_match),.I(FMU_L1A_MATCH));

	always @(posedge CLK40) begin
		LCT <= skw_rw_lct;
		case(TTC_SRC)
			FF_EMU_mode : begin
				L1A    		<= fmu_rw_l1a;
				L1A_MATCH   <= fmu_rw_l1a_match;
				RESYNC 		<= fmu_rw_resync;
			end
			FF_FEM_mode : begin
				L1A    		<= FEM_L1A;
				L1A_MATCH   <= FEM_L1A_MATCH;
				RESYNC 		<= FEM_RESYNC;
			end
			Skew_Clr_mode : begin
				L1A    		<= skw_rw_l1a;
				L1A_MATCH   <= skw_rw_l1a;
				RESYNC 		<= skw_rw_resync;
			end
			default: begin
				L1A    		<= fmu_rw_l1a;
				L1A_MATCH   <= fmu_rw_l1a_match;
				RESYNC 		<= fmu_rw_resync;
			end
		endcase
	end

endmodule
